package ru.myitschool.work.ui.main

import android.content.Context
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import createAuthHeader
import createRetrofit
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import kotlinx.serialization.json.JsonObject
import retrofit2.HttpException
import ru.myitschool.work.data.remote.ErrorDto
import ru.myitschool.work.data.remote.LoginApi
import ru.myitschool.work.data.remote.PersonInfoDto
import ru.myitschool.work.di.AppModule
import java.text.SimpleDateFormat
import javax.inject.Inject


@HiltViewModel
class MainViewModel @Inject constructor(
  @ApplicationContext private val context: Context,
  private val api: LoginApi,
  private val dataStoreManager: AppModule.DataStoreManager
) : ViewModel() {
  private val _state = MutableStateFlow(MainState())
  val state = _state.asStateFlow()

  private val dfo = SimpleDateFormat("yyyy-MM-dd HH:mm")
  private val dfi= SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")

  private val retrofit = createRetrofit("your_username", "your_password")
  private val loginApi = retrofit.create(LoginApi::class.java)

  fun loadPersonInfo(username: String, password : String) {
    viewModelScope.launch {
      try {
        val authHeader = createAuthHeader(username = username, password= password)
        val info = loginApi.info(authHeader, username)
        val personInfo = info.body()?.let { responseBody ->
          GsonBuilder().create().fromJson(responseBody.string(), PersonInfoDto::class.java)
        }
        println(personInfo)
        if (personInfo != null) {
            _state.update {
              MainState(
                fullName = personInfo.name,
                photo = personInfo.photo,
                position = personInfo.position,
                lastVisit = dfo.format(dfi.parse(personInfo.lastVisit)!!),
                role = personInfo.role,
                isBanned = personInfo.isBanned,
                error = null
              )
            }
        }
      } catch (httpException: HttpException) {
        try {
          httpException.response()?.errorBody()?.string()?.let { errorString ->
            val gson = GsonBuilder().create()
            val errorDto = gson.fromJson(errorString, ErrorDto::class.java)
            _state.update {
              MainState(
                error = errorDto.error
              )
            }
          }
        } catch (e: Exception) {
          _state.update { MainState(error = httpException.message()) }
        }
      } catch (e: Exception) {
        _state.update { MainState(error = "Unknown error: ${e.message}") }
      }
    }
  }


//  fun loadHistory(username: String, password : String): List<EntryClass> {
//    viewModelScope.launch {
//      try {
//        val authHeader = createAuthHeader(username = username, password= password)
//        val entries = loginApi.entries(authHeader, username)
////        val personEntries = entries.body()?.let { responseBody ->
////          GsonBuilder().create().fromJson(responseBody.string(), EntryClass::class.java)
////        }
////        val personEntries = Gson().fromJson()
//
//        val gson = Gson()
//        val listType = object : TypeToken<List<EntryClass>>() {}.type
//        val logEntries: List<EntryClass> = gson.fromJson(entries.body()?.string() , listType)
//
//
////        return logEntries
//      } catch (httpException: HttpException) {
//        try {
//          httpException.response()?.errorBody()?.string()?.let { errorString ->
//            val gson = GsonBuilder().create()
//            val errorDto = gson.fromJson(errorString, ErrorDto::class.java)
//            _state.update {
//              MainState(
//                error = errorDto.error
//              )
//            }
//          }
//        } catch (e: Exception) {
//          _state.update { MainState(error = httpException.message()) }
//        }
//      } catch (e: Exception) {
//        _state.update { MainState(error = "Unknown error: ${e.message}") }
//      }
//    }
//  }


  fun logout(onLogout: () -> Unit) {
    viewModelScope.launch {
      dataStoreManager.setLastUsername("")
      onLogout()
    }
  }
}